# A type of -*- python -*- file
# cython: language_level=3
import numpy as np
cimport numpy as np
cimport twophaseDarcyCoefficients as tpdc

cdef extern from "densityRelations.h":
    cdef cppclass ConstantDensity:
        pass
    cdef cppclass LinearDensity:
        pass
    cdef cppclass ExponentialDensity:
        pass
    cdef cppclass RealGasDensity:
        pass
    cdef cppclass IdealGasDensity:
        pass

cdef extern from "pskRelations.h":
    cdef cppclass SimplePSK:
        pass
    cdef cppclass PskSpline:
        pass
    cdef cppclass VGM:
        pass
    cdef cppclass VGB:
        pass
    cdef cppclass BCB:
        pass
    cdef cppclass BCM:
        pass

def twophaseDarcy_fc_sd_het_matType(int pskModelFlag,
                                    int density_w_flag,
                                    int density_n_flag,
                                    np.ndarray rowptr,
                                    np.ndarray colind,
                                    np.ndarray matType,
                                    double muw,
                                    double mun,
                                    np.ndarray omega,
                                    np.ndarray Kbar,
                                    double b,
                                    np.ndarray rwork_psk,
                                    np.ndarray iwork_psk,
                                    np.ndarray rwork_psk_tol,
                                    np.ndarray rwork_density_w,
                                    np.ndarray rwork_density_n,
                                    np.ndarray g,
                                    np.ndarray x,
                                    np.ndarray sw,
                                    np.ndarray psiw,
                                    np.ndarray mw,
                                    np.ndarray dmw,
                                    np.ndarray dmw_dpsiw,
                                    np.ndarray mn,
                                    np.ndarray dmn,
                                    np.ndarray dmn_dpsiw,
                                    np.ndarray psin,
                                    np.ndarray dpsin,
                                    np.ndarray dpsin_dpsiw,
                                    np.ndarray phi_psiw,
                                    np.ndarray dphi_psiw_dpsiw,
                                    np.ndarray phi_psin,
                                    np.ndarray dphi_psin_dpsiw,
                                    np.ndarray dphi_psin_dsw,
                                    np.ndarray aw,
                                    np.ndarray daw,
                                    np.ndarray daw_dpsiw,
                                    np.ndarray an,
                                    np.ndarray dan,
                                    np.ndarray dan_dpsiw):
    """Evaluate the coefficients of the fully coupled formulation of
    (slightly) compressible, two-phase Darcy flow for a heterogeneous
    medium, sparse diffusion rep. for het

    """
    cdef int nPointsPerSimplex=sw.shape[sw.ndim-1]
    cdef int nSimplex=sw.size//nPointsPerSimplex
    #while debugging allow only one option
    #exponential for wetting phase
    assert density_w_flag == 1
    if density_n_flag == 1: #nonwetting exponential
        if pskModelFlag == 0:
            tpdc.twophaseDarcy_fc_sd_het_matType[SimplePSK,ExponentialDensity,ExponentialDensity](nSimplex,
                                                                                                  nPointsPerSimplex,
                                                                                                  g.shape[0],
                                                                                                  2,
                                                                                                  <int*>(rowptr.data),
                                                                                                  <int*>(colind.data),
                                                                                                  <int*>(matType.data),
                                                                                                  muw,
                                                                                                  mun,
                                                                                                  <double*>(omega.data),
                                                                                                  <double*>(Kbar.data),
                                                                                                  b,
                                                                                                  <double*>(rwork_psk.data),
                                                                                                  <int*>(iwork_psk.data),
                                                                                                  <double*>(rwork_psk_tol.data),
                                                                                                  <double*>(rwork_density_w.data),
                                                                                                  <double*>(rwork_density_n.data),
                                                                                                  <double*>(g.data),
                                                                                                  <double*>(x.data),
                                                                                                  <double*>(sw.data),
                                                                                                  <double*>(psiw.data),
                                                                                                  <double*>(mw.data),
                                                                                                  <double*>(dmw.data),
                                                                                                  <double*>(dmw_dpsiw.data),
                                                                                                  <double*>(mn.data),
                                                                                                  <double*>(dmn.data),
                                                                                                  <double*>(dmn_dpsiw.data),
                                                                                                  <double*>(psin.data),
                                                                                                  <double*>(dpsin.data),
                                                                                                  <double*>(dpsin_dpsiw.data),
                                                                                                  <double*>(phi_psiw.data),
                                                                                                  <double*>(dphi_psiw_dpsiw.data),
                                                                                                  <double*>(phi_psin.data),
                                                                                                  <double*>(dphi_psin_dpsiw.data),
                                                                                                  <double*>(dphi_psin_dsw.data),
                                                                                                  <double*>(aw.data),
                                                                                                  <double*>(daw.data),
                                                                                                  <double*>(daw_dpsiw.data),
                                                                                                  <double*>(an.data),
                                                                                                  <double*>(dan.data),
                                                                                                  <double*>(dan_dpsiw.data))
        elif pskModelFlag == 1:
            tpdc.twophaseDarcy_fc_sd_het_matType[VGM,ExponentialDensity,ExponentialDensity](nSimplex,
                                                                                            nPointsPerSimplex,
                                                                                            g.shape[0],
                                                                                            4,
                                                                                            <int*>(rowptr.data),
                                                                                            <int*>(colind.data),
                                                                                            <int*>(matType.data),
                                                                                            muw,
                                                                                            mun,
                                                                                            <double*>(omega.data),
                                                                                            <double*>(Kbar.data),
                                                                                            b,
                                                                                            <double*>(rwork_psk.data),
                                                                                            <int*>(iwork_psk.data),
                                                                                            <double*>(rwork_psk_tol.data),
                                                                                            <double*>(rwork_density_w.data),
                                                                                            <double*>(rwork_density_n.data),
                                                                                            <double*>(g.data),
                                                                                            <double*>(x.data),
                                                                                            <double*>(sw.data),
                                                                                            <double*>(psiw.data),
                                                                                            <double*>(mw.data),
                                                                                            <double*>(dmw.data),
                                                                                            <double*>(dmw_dpsiw.data),
                                                                                            <double*>(mn.data),
                                                                                            <double*>(dmn.data),
                                                                                            <double*>(dmn_dpsiw.data),
                                                                                            <double*>(psin.data),
                                                                                            <double*>(dpsin.data),
                                                                                            <double*>(dpsin_dpsiw.data),
                                                                                            <double*>(phi_psiw.data),
                                                                                            <double*>(dphi_psiw_dpsiw.data),
                                                                                            <double*>(phi_psin.data),
                                                                                            <double*>(dphi_psin_dpsiw.data),
                                                                                            <double*>(dphi_psin_dsw.data),
                                                                                            <double*>(aw.data),
                                                                                            <double*>(daw.data),
                                                                                            <double*>(daw_dpsiw.data),
                                                                                            <double*>(an.data),
                                                                                            <double*>(dan.data),
                                                                                            <double*>(dan_dpsiw.data))
        elif pskModelFlag == 5:
            tpdc.twophaseDarcy_fc_sd_het_matType[PskSpline,ExponentialDensity,ExponentialDensity](nSimplex,
                                                                                                  nPointsPerSimplex,
                                                                                                  g.shape[0],
                                                                                                  iwork_psk[1],
                                                                                                  <int*>(rowptr.data),
                                                                                                  <int*>(colind.data),
                                                                                                  <int*>(matType.data),
                                                                                                  muw,
                                                                                                  mun,
                                                                                                  <double*>(omega.data),
                                                                                                  <double*>(Kbar.data),
                                                                                                  b,
                                                                                                  <double*>(rwork_psk.data),
                                                                                                  <int*>(iwork_psk.data),
                                                                                                  <double*>(rwork_psk_tol.data),
                                                                                                  <double*>(rwork_density_w.data),
                                                                                                  <double*>(rwork_density_n.data),
                                                                                                  <double*>(g.data),
                                                                                                  <double*>(x.data),
                                                                                                  <double*>(sw.data),
                                                                                                  <double*>(psiw.data),
                                                                                                  <double*>(mw.data),
                                                                                                  <double*>(dmw.data),
                                                                                                  <double*>(dmw_dpsiw.data),
                                                                                                  <double*>(mn.data),
                                                                                                  <double*>(dmn.data),
                                                                                                  <double*>(dmn_dpsiw.data),
                                                                                                  <double*>(psin.data),
                                                                                                  <double*>(dpsin.data),
                                                                                                  <double*>(dpsin_dpsiw.data),
                                                                                                  <double*>(phi_psiw.data),
                                                                                                  <double*>(dphi_psiw_dpsiw.data),
                                                                                                  <double*>(phi_psin.data),
                                                                                                  <double*>(dphi_psin_dpsiw.data),
                                                                                                  <double*>(dphi_psin_dsw.data),
                                                                                                  <double*>(aw.data),
                                                                                                  <double*>(daw.data),
                                                                                                  <double*>(daw_dpsiw.data),
                                                                                                  <double*>(an.data),
                                                                                                  <double*>(dan.data),
                                                                                                  <double*>(dan_dpsiw.data))
        else:
            tpdc.twophaseDarcy_fc_sd_het_matType[SimplePSK,ExponentialDensity,ExponentialDensity](nSimplex,
                                                                                                  nPointsPerSimplex,
                                                                                                  g.shape[0],
                                                                                                  2,
                                                                                                  <int*>(rowptr.data),
                                                                                                  <int*>(colind.data),
                                                                                                  <int*>(matType.data),
                                                                                                  muw,
                                                                                                  mun,
                                                                                                  <double*>(omega.data),
                                                                                                  <double*>(Kbar.data),
                                                                                                  b,
                                                                                                  <double*>(rwork_psk.data),
                                                                                                  <int*>(iwork_psk.data),
                                                                                                  <double*>(rwork_psk_tol.data),
                                                                                                  <double*>(rwork_density_w.data),
                                                                                                  <double*>(rwork_density_n.data),
                                                                                                  <double*>(g.data),
                                                                                                  <double*>(x.data),
                                                                                                  <double*>(sw.data),
                                                                                                  <double*>(psiw.data),
                                                                                                  <double*>(mw.data),
                                                                                                  <double*>(dmw.data),
                                                                                                  <double*>(dmw_dpsiw.data),
                                                                                                  <double*>(mn.data),
                                                                                                  <double*>(dmn.data),
                                                                                                  <double*>(dmn_dpsiw.data),
                                                                                                  <double*>(psin.data),
                                                                                                  <double*>(dpsin.data),
                                                                                                  <double*>(dpsin_dpsiw.data),
                                                                                                  <double*>(phi_psiw.data),
                                                                                                  <double*>(dphi_psiw_dpsiw.data),
                                                                                                  <double*>(phi_psin.data),
                                                                                                  <double*>(dphi_psin_dpsiw.data),
                                                                                                  <double*>(dphi_psin_dsw.data),
                                                                                                  <double*>(aw.data),
                                                                                                  <double*>(daw.data),
                                                                                                  <double*>(daw_dpsiw.data),
                                                                                                  <double*>(an.data),
                                                                                                  <double*>(dan.data),
                                                                                                  <double*>(dan_dpsiw.data))
    elif density_n_flag == 2:#nonwetting IdealGas
        if pskModelFlag == 0:
            tpdc.twophaseDarcy_fc_sd_het_matType[SimplePSK,ExponentialDensity,IdealGasDensity](nSimplex,
                                                                                               nPointsPerSimplex,
                                                                                               g.shape[0],
                                                                                               2,
                                                                                               <int*>(rowptr.data),
                                                                                               <int*>(colind.data),
                                                                                               <int*>(matType.data),
                                                                                               muw,
                                                                                               mun,
                                                                                               <double*>(omega.data),
                                                                                               <double*>(Kbar.data),
                                                                                               b,
                                                                                               <double*>(rwork_psk.data),
                                                                                               <int*>(iwork_psk.data),
                                                                                               <double*>(rwork_psk_tol.data),
                                                                                               <double*>(rwork_density_w.data),
                                                                                               <double*>(rwork_density_n.data),
                                                                                               <double*>(g.data),
                                                                                               <double*>(x.data),
                                                                                               <double*>(sw.data),
                                                                                               <double*>(psiw.data),
                                                                                               <double*>(mw.data),
                                                                                               <double*>(dmw.data),
                                                                                               <double*>(dmw_dpsiw.data),
                                                                                               <double*>(mn.data),
                                                                                               <double*>(dmn.data),
                                                                                               <double*>(dmn_dpsiw.data),
                                                                                               <double*>(psin.data),
                                                                                               <double*>(dpsin.data),
                                                                                               <double*>(dpsin_dpsiw.data),
                                                                                               <double*>(phi_psiw.data),
                                                                                               <double*>(dphi_psiw_dpsiw.data),
                                                                                               <double*>(phi_psin.data),
                                                                                               <double*>(dphi_psin_dpsiw.data),
                                                                                               <double*>(dphi_psin_dsw.data),
                                                                                               <double*>(aw.data),
                                                                                               <double*>(daw.data),
                                                                                               <double*>(daw_dpsiw.data),
                                                                                               <double*>(an.data),
                                                                                               <double*>(dan.data),
                                                                                               <double*>(dan_dpsiw.data))
        elif pskModelFlag == 1:
            tpdc.twophaseDarcy_fc_sd_het_matType[VGM,ExponentialDensity,IdealGasDensity](nSimplex,
                                                                                         nPointsPerSimplex,
                                                                                         g.shape[0],
                                                                                         4,
                                                                                         <int*>(rowptr.data),
                                                                                         <int*>(colind.data),
                                                                                         <int*>(matType.data),
                                                                                         muw,
                                                                                         mun,
                                                                                         <double*>(omega.data),
                                                                                         <double*>(Kbar.data),
                                                                                         b,
                                                                                         <double*>(rwork_psk.data),
                                                                                         <int*>(iwork_psk.data),
                                                                                         <double*>(rwork_psk_tol.data),
                                                                                         <double*>(rwork_density_w.data),
                                                                                         <double*>(rwork_density_n.data),
                                                                                         <double*>(g.data),
                                                                                         <double*>(x.data),
                                                                                         <double*>(sw.data),
                                                                                         <double*>(psiw.data),
                                                                                         <double*>(mw.data),
                                                                                         <double*>(dmw.data),
                                                                                         <double*>(dmw_dpsiw.data),
                                                                                         <double*>(mn.data),
                                                                                         <double*>(dmn.data),
                                                                                         <double*>(dmn_dpsiw.data),
                                                                                         <double*>(psin.data),
                                                                                         <double*>(dpsin.data),
                                                                                         <double*>(dpsin_dpsiw.data),
                                                                                         <double*>(phi_psiw.data),
                                                                                         <double*>(dphi_psiw_dpsiw.data),
                                                                                         <double*>(phi_psin.data),
                                                                                         <double*>(dphi_psin_dpsiw.data),
                                                                                         <double*>(dphi_psin_dsw.data),
                                                                                         <double*>(aw.data),
                                                                                         <double*>(daw.data),
                                                                                         <double*>(daw_dpsiw.data),
                                                                                         <double*>(an.data),
                                                                                         <double*>(dan.data),
                                                                                         <double*>(dan_dpsiw.data))
        elif pskModelFlag == 5:
            tpdc.twophaseDarcy_fc_sd_het_matType[PskSpline,ExponentialDensity,IdealGasDensity](nSimplex,
                                                                                               nPointsPerSimplex,
                                                                                               g.shape[0],
                                                                                               iwork_psk[1],
                                                                                               <int*>(rowptr.data),
                                                                                               <int*>(colind.data),
                                                                                               <int*>(matType.data),
                                                                                               muw,
                                                                                               mun,
                                                                                               <double*>(omega.data),
                                                                                               <double*>(Kbar.data),
                                                                                               b,
                                                                                               <double*>(rwork_psk.data),
                                                                                               <int*>(iwork_psk.data),
                                                                                               <double*>(rwork_psk_tol.data),
                                                                                               <double*>(rwork_density_w.data),
                                                                                               <double*>(rwork_density_n.data),
                                                                                               <double*>(g.data),
                                                                                               <double*>(x.data),
                                                                                               <double*>(sw.data),
                                                                                               <double*>(psiw.data),
                                                                                               <double*>(mw.data),
                                                                                               <double*>(dmw.data),
                                                                                               <double*>(dmw_dpsiw.data),
                                                                                               <double*>(mn.data),
                                                                                               <double*>(dmn.data),
                                                                                               <double*>(dmn_dpsiw.data),
                                                                                               <double*>(psin.data),
                                                                                               <double*>(dpsin.data),
                                                                                               <double*>(dpsin_dpsiw.data),
                                                                                               <double*>(phi_psiw.data),
                                                                                               <double*>(dphi_psiw_dpsiw.data),
                                                                                               <double*>(phi_psin.data),
                                                                                               <double*>(dphi_psin_dpsiw.data),
                                                                                               <double*>(dphi_psin_dsw.data),
                                                                                               <double*>(aw.data),
                                                                                               <double*>(daw.data),
                                                                                               <double*>(daw_dpsiw.data),
                                                                                               <double*>(an.data),
                                                                                               <double*>(dan.data),
                                                                                               <double*>(dan_dpsiw.data))
        else:
            tpdc.twophaseDarcy_fc_sd_het_matType[SimplePSK,ExponentialDensity,IdealGasDensity](nSimplex,
                                                                                               nPointsPerSimplex,
                                                                                               g.shape[0],
                                                                                               2,
                                                                                               <int*>(rowptr.data),
                                                                                               <int*>(colind.data),
                                                                                               <int*>(matType.data),
                                                                                               muw,
                                                                                               mun,
                                                                                               <double*>(omega.data),
                                                                                               <double*>(Kbar.data),
                                                                                               b,
                                                                                               <double*>(rwork_psk.data),
                                                                                               <int*>(iwork_psk.data),
                                                                                               <double*>(rwork_psk_tol.data),
                                                                                               <double*>(rwork_density_w.data),
                                                                                               <double*>(rwork_density_n.data),
                                                                                               <double*>(g.data),
                                                                                               <double*>(x.data),
                                                                                               <double*>(sw.data),
                                                                                               <double*>(psiw.data),
                                                                                               <double*>(mw.data),
                                                                                               <double*>(dmw.data),
                                                                                               <double*>(dmw_dpsiw.data),
                                                                                               <double*>(mn.data),
                                                                                               <double*>(dmn.data),
                                                                                               <double*>(dmn_dpsiw.data),
                                                                                               <double*>(psin.data),
                                                                                               <double*>(dpsin.data),
                                                                                               <double*>(dpsin_dpsiw.data),
                                                                                               <double*>(phi_psiw.data),
                                                                                               <double*>(dphi_psiw_dpsiw.data),
                                                                                               <double*>(phi_psin.data),
                                                                                               <double*>(dphi_psin_dpsiw.data),
                                                                                               <double*>(dphi_psin_dsw.data),
                                                                                               <double*>(aw.data),
                                                                                               <double*>(daw.data),
                                                                                               <double*>(daw_dpsiw.data),
                                                                                               <double*>(an.data),
                                                                                               <double*>(dan.data),
                                                                                               <double*>(dan_dpsiw.data))
    else:
        assert False, "configuration density_w_flag= {0} and density_n_flag= {1} not found ".format(density_w_flag, density_n_flag)

def twophaseDarcy_fc_sd_het_matType_nonPotentialForm(int pskModelFlag,
                                                     int density_w_flag,
                                                     int density_n_flag,
                                                     np.ndarray rowptr,
                                                     np.ndarray colind,
                                                     np.ndarray matType,
                                                     double muw,
                                                     double mun,
                                                     np.ndarray omega,
                                                     np.ndarray Kbar,
                                                     double b,
                                                     np.ndarray rwork_psk,
                                                     np.ndarray iwork_psk,
                                                     np.ndarray rwork_psk_tol,
                                                     np.ndarray rwork_density_w,
                                                     np.ndarray rwork_density_n,
                                                     np.ndarray g,
                                                     np.ndarray x,
                                                     np.ndarray sw,
                                                     np.ndarray psiw,
                                                     np.ndarray mw,
                                                     np.ndarray dmw,
                                                     np.ndarray dmw_dpsiw,
                                                     np.ndarray mn,
                                                     np.ndarray dmn,
                                                     np.ndarray dmn_dpsiw,
                                                     np.ndarray psin,
                                                     np.ndarray dpsin,
                                                     np.ndarray dpsin_dpsiw,
                                                     np.ndarray phi_psiw,
                                                     np.ndarray dphi_psiw_dpsiw,
                                                     np.ndarray phi_psin,
                                                     np.ndarray dphi_psin_dpsiw,
                                                     np.ndarray dphi_psin_dsw,
                                                     np.ndarray fw,
                                                     np.ndarray dfw_dsw,
                                                     np.ndarray dfw_dpsiw,
                                                     np.ndarray fn,
                                                     np.ndarray dfn_dsw,
                                                     np.ndarray dfn_dpsiw,
                                                     np.ndarray aw,
                                                     np.ndarray daw,
                                                     np.ndarray daw_dpsiw,
                                                     np.ndarray an,
                                                     np.ndarray dan,
                                                     np.ndarray dan_dpsiw,
                                                     int compressibilityFlag):
    """"Evaluate the coefficients of the fully coupled formulation of
    (slightly) compressible, two-phase Darcy flow for a heterogeneous
    medium, sparse diffusion rep. for het

    """
    cdef int nPointsPerSimplex=sw.shape[sw.ndim-1]
    cdef int nSimplex=sw.size//nPointsPerSimplex
    assert density_w_flag == 1
    if density_n_flag == 1:#nonwetting exponential
        if pskModelFlag == 0:
            tpdc.twophaseDarcy_fc_sd_het_matType_nonPotentialForm[SimplePSK,ExponentialDensity,ExponentialDensity](compressibilityFlag,nSimplex,
                                                                                                                   nPointsPerSimplex,
                                                                                                                   g.shape[0],
                                                                                                                   2,
                                                                                                                   <int*>(rowptr.data),
                                                                                                                   <int*>(colind.data),
                                                                                                                   <int*>(matType.data),
                                                                                                                   muw,
                                                                                                                   mun,
                                                                                                                   <double*>(omega.data),
                                                                                                                   <double*>(Kbar.data),
                                                                                                                   b,
                                                                                                                   <double*>(rwork_psk.data),
                                                                                                                   <int*>(iwork_psk.data),
                                                                                                                   <double*>(rwork_psk_tol.data),
                                                                                                                   <double*>(rwork_density_w.data),
                                                                                                                   <double*>(rwork_density_n.data),
                                                                                                                   <double*>(g.data),
                                                                                                                   <double*>(x.data),
                                                                                                                   <double*>(sw.data),
                                                                                                                   <double*>(psiw.data),
                                                                                                                   <double*>(mw.data),
                                                                                                                   <double*>(dmw.data),
                                                                                                                   <double*>(dmw_dpsiw.data),
                                                                                                                   <double*>(mn.data),
                                                                                                                   <double*>(dmn.data),
                                                                                                                   <double*>(dmn_dpsiw.data),
                                                                                                                   <double*>(psin.data),
                                                                                                                   <double*>(dpsin.data),
                                                                                                                   <double*>(dpsin_dpsiw.data),
                                                                                                                   <double*>(phi_psiw.data),
                                                                                                                   <double*>(dphi_psiw_dpsiw.data),
                                                                                                                   <double*>(phi_psin.data),
                                                                                                                   <double*>(dphi_psin_dpsiw.data),
                                                                                                                   <double*>(dphi_psin_dsw.data),
                                                                                                                   <double*>(fw.data),
                                                                                                                   <double*>(dfw_dsw.data),
                                                                                                                   <double*>(dfw_dpsiw.data),
                                                                                                                   <double*>(fn.data),
                                                                                                                   <double*>(dfn_dsw.data),
                                                                                                                   <double*>(dfn_dpsiw.data),
                                                                                                                   <double*>(aw.data),
                                                                                                                   <double*>(daw.data),
                                                                                                                   <double*>(daw_dpsiw.data),
                                                                                                                   <double*>(an.data),
                                                                                                                   <double*>(dan.data),
                                                                                                                   <double*>(dan_dpsiw.data))
        elif pskModelFlag == 1:
            tpdc.twophaseDarcy_fc_sd_het_matType_nonPotentialForm[VGM,ExponentialDensity,ExponentialDensity](compressibilityFlag,nSimplex,
                                                                                                             nPointsPerSimplex,
                                                                                                             g.shape[0],
                                                                                                             4,
                                                                                                             <int*>(rowptr.data),
                                                                                                             <int*>(colind.data),
                                                                                                             <int*>(matType.data),
                                                                                                             muw,
                                                                                                             mun,
                                                                                                             <double*>(omega.data),
                                                                                                             <double*>(Kbar.data),
                                                                                                             b,
                                                                                                             <double*>(rwork_psk.data),
                                                                                                             <int*>(iwork_psk.data),
                                                                                                             <double*>(rwork_psk_tol.data),
                                                                                                             <double*>(rwork_density_w.data),
                                                                                                             <double*>(rwork_density_n.data),
                                                                                                             <double*>(g.data),
                                                                                                             <double*>(x.data),
                                                                                                             <double*>(sw.data),
                                                                                                             <double*>(psiw.data),
                                                                                                             <double*>(mw.data),
                                                                                                             <double*>(dmw.data),
                                                                                                             <double*>(dmw_dpsiw.data),
                                                                                                             <double*>(mn.data),
                                                                                                             <double*>(dmn.data),
                                                                                                             <double*>(dmn_dpsiw.data),
                                                                                                             <double*>(psin.data),
                                                                                                             <double*>(dpsin.data),
                                                                                                             <double*>(dpsin_dpsiw.data),
                                                                                                             <double*>(phi_psiw.data),
                                                                                                             <double*>(dphi_psiw_dpsiw.data),
                                                                                                             <double*>(phi_psin.data),
                                                                                                             <double*>(dphi_psin_dpsiw.data),
                                                                                                             <double*>(dphi_psin_dsw.data),
                                                                                                             <double*>(fw.data),
                                                                                                             <double*>(dfw_dsw.data),
                                                                                                             <double*>(dfw_dpsiw.data),
                                                                                                             <double*>(fn.data),
                                                                                                             <double*>(dfn_dsw.data),
                                                                                                             <double*>(dfn_dpsiw.data),
                                                                                                             <double*>(aw.data),
                                                                                                             <double*>(daw.data),
                                                                                                             <double*>(daw_dpsiw.data),
                                                                                                             <double*>(an.data),
                                                                                                             <double*>(dan.data),
                                                                                                             <double*>(dan_dpsiw.data))
        else:
            tpdc.twophaseDarcy_fc_sd_het_matType_nonPotentialForm[SimplePSK,ExponentialDensity,ExponentialDensity](compressibilityFlag,nSimplex,
                                                                                                                   nPointsPerSimplex,
                                                                                                                   g.shape[0],
                                                                                                                   2,
                                                                                                                   <int*>(rowptr.data),
                                                                                                                   <int*>(colind.data),
                                                                                                                   <int*>(matType.data),
                                                                                                                   muw,
                                                                                                                   mun,
                                                                                                                   <double*>(omega.data),
                                                                                                                   <double*>(Kbar.data),
                                                                                                                   b,
                                                                                                                   <double*>(rwork_psk.data),
                                                                                                                   <int*>(iwork_psk.data),
                                                                                                                   <double*>(rwork_psk_tol.data),
                                                                                                                   <double*>(rwork_density_w.data),
                                                                                                                   <double*>(rwork_density_n.data),
                                                                                                                   <double*>(g.data),
                                                                                                                   <double*>(x.data),
                                                                                                                   <double*>(sw.data),
                                                                                                                   <double*>(psiw.data),
                                                                                                                   <double*>(mw.data),
                                                                                                                   <double*>(dmw.data),
                                                                                                                   <double*>(dmw_dpsiw.data),
                                                                                                                   <double*>(mn.data),
                                                                                                                   <double*>(dmn.data),
                                                                                                                   <double*>(dmn_dpsiw.data),
                                                                                                                   <double*>(psin.data),
                                                                                                                   <double*>(dpsin.data),
                                                                                                                   <double*>(dpsin_dpsiw.data),
                                                                                                                   <double*>(phi_psiw.data),
                                                                                                                   <double*>(dphi_psiw_dpsiw.data),
                                                                                                                   <double*>(phi_psin.data),
                                                                                                                   <double*>(dphi_psin_dpsiw.data),
                                                                                                                   <double*>(dphi_psin_dsw.data),
                                                                                                                   <double*>(fw.data),
                                                                                                                   <double*>(dfw_dsw.data),
                                                                                                                   <double*>(dfw_dpsiw.data),
                                                                                                                   <double*>(fn.data),
                                                                                                                   <double*>(dfn_dsw.data),
                                                                                                                   <double*>(dfn_dpsiw.data),
                                                                                                                   <double*>(aw.data),
                                                                                                                   <double*>(daw.data),
                                                                                                                   <double*>(daw_dpsiw.data),
                                                                                                                   <double*>(an.data),
                                                                                                                   <double*>(dan.data),
                                                                                                                   <double*>(dan_dpsiw.data))
    elif density_n_flag == 2:#nonwetting IdealGas
        if pskModelFlag == 1:
            tpdc.twophaseDarcy_fc_sd_het_matType_nonPotentialForm[VGM,ExponentialDensity,IdealGasDensity](compressibilityFlag,nSimplex,
                                                                                                          nPointsPerSimplex,
                                                                                                          g.shape[0],
                                                                                                          4,
                                                                                                          <int*>(rowptr.data),
                                                                                                          <int*>(colind.data),
                                                                                                          <int*>(matType.data),
                                                                                                          muw,
                                                                                                          mun,
                                                                                                          <double*>(omega.data),
                                                                                                          <double*>(Kbar.data),
                                                                                                          b,
                                                                                                          <double*>(rwork_psk.data),
                                                                                                          <int*>(iwork_psk.data),
                                                                                                          <double*>(rwork_psk_tol.data),
                                                                                                          <double*>(rwork_density_w.data),
                                                                                                          <double*>(rwork_density_n.data),
                                                                                                          <double*>(g.data),
                                                                                                          <double*>(x.data),
                                                                                                          <double*>(sw.data),
                                                                                                          <double*>(psiw.data),
                                                                                                          <double*>(mw.data),
                                                                                                          <double*>(dmw.data),
                                                                                                          <double*>(dmw_dpsiw.data),
                                                                                                          <double*>(mn.data),
                                                                                                          <double*>(dmn.data),
                                                                                                          <double*>(dmn_dpsiw.data),
                                                                                                          <double*>(psin.data),
                                                                                                          <double*>(dpsin.data),
                                                                                                          <double*>(dpsin_dpsiw.data),
                                                                                                          <double*>(phi_psiw.data),
                                                                                                          <double*>(dphi_psiw_dpsiw.data),
                                                                                                          <double*>(phi_psin.data),
                                                                                                          <double*>(dphi_psin_dpsiw.data),
                                                                                                          <double*>(dphi_psin_dsw.data),
                                                                                                          <double*>(fw.data),
                                                                                                          <double*>(dfw_dsw.data),
                                                                                                          <double*>(dfw_dpsiw.data),
                                                                                                          <double*>(fn.data),
                                                                                                          <double*>(dfn_dsw.data),
                                                                                                          <double*>(dfn_dpsiw.data),
                                                                                                          <double*>(aw.data),
                                                                                                          <double*>(daw.data),
                                                                                                          <double*>(daw_dpsiw.data),
                                                                                                          <double*>(an.data),
                                                                                                          <double*>(dan.data),
                                                                                                          <double*>(dan_dpsiw.data))
        else:
            tpdc.twophaseDarcy_fc_sd_het_matType_nonPotentialForm[SimplePSK,ExponentialDensity,IdealGasDensity](compressibilityFlag,nSimplex,
                                                                                                                nPointsPerSimplex,
                                                                                                                g.shape[0],
                                                                                                                2,
                                                                                                                <int*>(rowptr.data),
                                                                                                                <int*>(colind.data),
                                                                                                                <int*>(matType.data),
                                                                                                                muw,
                                                                                                                mun,
                                                                                                                <double*>(omega.data),
                                                                                                                <double*>(Kbar.data),
                                                                                                                b,
                                                                                                                <double*>(rwork_psk.data),
                                                                                                                <int*>(iwork_psk.data),
                                                                                                                <double*>(rwork_psk_tol.data),
                                                                                                                <double*>(rwork_density_w.data),
                                                                                                                <double*>(rwork_density_n.data),
                                                                                                                <double*>(g.data),
                                                                                                                <double*>(x.data),
                                                                                                                <double*>(sw.data),
                                                                                                                <double*>(psiw.data),
                                                                                                                <double*>(mw.data),
                                                                                                                <double*>(dmw.data),
                                                                                                                <double*>(dmw_dpsiw.data),
                                                                                                                <double*>(mn.data),
                                                                                                                <double*>(dmn.data),
                                                                                                                <double*>(dmn_dpsiw.data),
                                                                                                                <double*>(psin.data),
                                                                                                                <double*>(dpsin.data),
                                                                                                                <double*>(dpsin_dpsiw.data),
                                                                                                                <double*>(phi_psiw.data),
                                                                                                                <double*>(dphi_psiw_dpsiw.data),
                                                                                                                <double*>(phi_psin.data),
                                                                                                                <double*>(dphi_psin_dpsiw.data),
                                                                                                                <double*>(dphi_psin_dsw.data),
                                                                                                                <double*>(fw.data),
                                                                                                                <double*>(dfw_dsw.data),
                                                                                                                <double*>(dfw_dpsiw.data),
                                                                                                                <double*>(fn.data),
                                                                                                                <double*>(dfn_dsw.data),
                                                                                                                <double*>(dfn_dpsiw.data),
                                                                                                                <double*>(aw.data),
                                                                                                                <double*>(daw.data),
                                                                                                                <double*>(daw_dpsiw.data),
                                                                                                                <double*>(an.data),
                                                                                                                <double*>(dan.data),
                                                                                                                <double*>(dan_dpsiw.data))
    else:
        assert False, "configuration density_w_flag= {0} and density_n_flag= {1} not found ".format(density_w_flag, density_n_flag)

def twophaseDarcy_fc_pp_sd_het_matType(int pskModelFlag,
                                       int density_w_flag,
                                       int density_n_flag,
                                       np.ndarray rowptr,
                                       np.ndarray colind,
                                       np.ndarray matType,
                                       double muw,
                                       double mun,
                                       np.ndarray omega,
                                       np.ndarray Kbar,
                                       double b,
                                       np.ndarray rwork_psk,
                                       np.ndarray iwork_psk,
                                       np.ndarray rwork_psk_tol,
                                       np.ndarray rwork_density_w,
                                       np.ndarray rwork_density_n,
                                       np.ndarray g,
                                       np.ndarray x,
                                       np.ndarray psiw,
                                       np.ndarray psic,
                                       np.ndarray sw,
                                       np.ndarray mw,
                                       np.ndarray dmw_dpsiw,
                                       np.ndarray dmw_dpsic,
                                       np.ndarray mn,
                                       np.ndarray dmn_dpsiw,
                                       np.ndarray dmn_dpsic,
                                       np.ndarray phi_psiw,
                                       np.ndarray dphi_psiw_dpsiw,
                                       np.ndarray phi_psin,
                                       np.ndarray dphi_psin_dpsiw,
                                       np.ndarray dphi_psin_dpsic,
                                       np.ndarray aw,
                                       np.ndarray daw_dpsiw,
                                       np.ndarray daw_dpsic,
                                       np.ndarray an,
                                       np.ndarray dan_dpsiw,
                                       np.ndarray dan_dpsic):
    """Evaluate the coefficients of the fully coupled formulation of
    (slightly) compressible, two-phase Darcy flow for a heterogeneous
    medium, pressure-pressure sparse diffusion rep. for het

    """
    cdef int nPointsPerSimplex=sw.shape[sw.ndim-1]
    cdef int nSimplex=sw.size//nPointsPerSimplex
    #while debugging allow only one option
    #exponential for wetting phase
    assert density_w_flag == 1
    if density_n_flag == 1: #nonwetting exponential
        if pskModelFlag == 0:
            tpdc.twophaseDarcy_fc_pp_sd_het_matType[SimplePSK,ExponentialDensity,ExponentialDensity](nSimplex,
                                                                                                nPointsPerSimplex,
                                                                                                g.shape[0],
                                                                                                2,
                                                                                                <int*>(rowptr.data),
                                                                                                <int*>(colind.data),
                                                                                                <int*>(matType.data),
                                                                                                muw,
                                                                                                mun,
                                                                                                <double*>(omega.data),
                                                                                                <double*>(Kbar.data),
                                                                                                b,
                                                                                                <double*>(rwork_psk.data),
                                                                                                <int*>(iwork_psk.data),
                                                                                                <double*>(rwork_psk_tol.data),
                                                                                                <double*>(rwork_density_w.data),
                                                                                                <double*>(rwork_density_n.data),
                                                                                                <double*>(g.data),
                                                                                                <double*>(x.data),
                                                                                                <double*>(psiw.data),
                                                                                                <double*>(psic.data),
                                                                                                <double*>(sw.data),
                                                                                                <double*>(mw.data),
                                                                                                <double*>(dmw_dpsiw.data),
                                                                                                <double*>(dmw_dpsic.data),
                                                                                                <double*>(mn.data),
                                                                                                <double*>(dmn_dpsiw.data),
                                                                                                <double*>(dmn_dpsic.data),
                                                                                                <double*>(phi_psiw.data),
                                                                                                <double*>(dphi_psiw_dpsiw.data),
                                                                                                <double*>(phi_psin.data),
                                                                                                <double*>(dphi_psin_dpsiw.data),
                                                                                                <double*>(dphi_psin_dpsic.data),
                                                                                                <double*>(aw.data),
                                                                                                <double*>(daw_dpsiw.data),
                                                                                                <double*>(daw_dpsic.data),
                                                                                                <double*>(an.data),
                                                                                                <double*>(dan_dpsiw.data),
                                                                                                <double*>(dan_dpsic.data))
        elif pskModelFlag == 1:
            tpdc.twophaseDarcy_fc_pp_sd_het_matType[VGM,ExponentialDensity,ExponentialDensity](nSimplex,
                                                                                          nPointsPerSimplex,
                                                                                          g.shape[0],
                                                                                          4,
                                                                                          <int*>(rowptr.data),
                                                                                          <int*>(colind.data),
                                                                                          <int*>(matType.data),
                                                                                          muw,
                                                                                          mun,
                                                                                          <double*>(omega.data),
                                                                                          <double*>(Kbar.data),
                                                                                          b,
                                                                                          <double*>(rwork_psk.data),
                                                                                          <int*>(iwork_psk.data),
                                                                                          <double*>(rwork_psk_tol.data),
                                                                                          <double*>(rwork_density_w.data),
                                                                                          <double*>(rwork_density_n.data),
                                                                                          <double*>(g.data),
                                                                                          <double*>(x.data),
                                                                                          <double*>(psiw.data),
                                                                                          <double*>(psic.data),
                                                                                          <double*>(sw.data),
                                                                                          <double*>(mw.data),
                                                                                          <double*>(dmw_dpsiw.data),
                                                                                          <double*>(dmw_dpsic.data),
                                                                                          <double*>(mn.data),
                                                                                          <double*>(dmn_dpsiw.data),
                                                                                          <double*>(dmn_dpsic.data),
                                                                                          <double*>(phi_psiw.data),
                                                                                          <double*>(dphi_psiw_dpsiw.data),
                                                                                          <double*>(phi_psin.data),
                                                                                          <double*>(dphi_psin_dpsiw.data),
                                                                                          <double*>(dphi_psin_dpsic.data),
                                                                                          <double*>(aw.data),
                                                                                          <double*>(daw_dpsiw.data),
                                                                                          <double*>(daw_dpsic.data),
                                                                                          <double*>(an.data),
                                                                                          <double*>(dan_dpsiw.data),
                                                                                          <double*>(dan_dpsic.data))
        else:
            tpdc.twophaseDarcy_fc_pp_sd_het_matType[SimplePSK,ExponentialDensity,ExponentialDensity](nSimplex,
                                                                                                nPointsPerSimplex,
                                                                                                g.shape[0],
                                                                                                2,
                                                                                                <int*>(rowptr.data),
                                                                                                <int*>(colind.data),
                                                                                                <int*>(matType.data),
                                                                                                muw,
                                                                                                mun,
                                                                                                <double*>(omega.data),
                                                                                                <double*>(Kbar.data),
                                                                                                b,
                                                                                                <double*>(rwork_psk.data),
                                                                                                <int*>(iwork_psk.data),
                                                                                                <double*>(rwork_psk_tol.data),
                                                                                                <double*>(rwork_density_w.data),
                                                                                                <double*>(rwork_density_n.data),
                                                                                                <double*>(g.data),
                                                                                                <double*>(x.data),
                                                                                                <double*>(psiw.data),
                                                                                                <double*>(psic.data),
                                                                                                <double*>(sw.data),
                                                                                                <double*>(mw.data),
                                                                                                <double*>(dmw_dpsiw.data),
                                                                                                <double*>(dmw_dpsic.data),
                                                                                                <double*>(mn.data),
                                                                                                <double*>(dmn_dpsiw.data),
                                                                                                <double*>(dmn_dpsic.data),
                                                                                                <double*>(phi_psiw.data),
                                                                                                <double*>(dphi_psiw_dpsiw.data),
                                                                                                <double*>(phi_psin.data),
                                                                                                <double*>(dphi_psin_dpsiw.data),
                                                                                                <double*>(dphi_psin_dpsic.data),
                                                                                                <double*>(aw.data),
                                                                                                <double*>(daw_dpsiw.data),
                                                                                                <double*>(daw_dpsic.data),
                                                                                                <double*>(an.data),
                                                                                                <double*>(dan_dpsiw.data),
                                                                                                <double*>(dan_dpsic.data))
    elif density_n_flag == 2: #nonwetting IdealGas
        if pskModelFlag == 0:
            tpdc.twophaseDarcy_fc_pp_sd_het_matType[SimplePSK,ExponentialDensity,IdealGasDensity](nSimplex,
                                                                                             nPointsPerSimplex,
                                                                                             g.shape[0],
                                                                                             2,
                                                                                             <int*>(rowptr.data),
                                                                                             <int*>(colind.data),
                                                                                             <int*>(matType.data),
                                                                                             muw,
                                                                                             mun,
                                                                                             <double*>(omega.data),
                                                                                             <double*>(Kbar.data),
                                                                                             b,
                                                                                             <double*>(rwork_psk.data),
                                                                                             <int*>(iwork_psk.data),
                                                                                             <double*>(rwork_psk_tol.data),
                                                                                             <double*>(rwork_density_w.data),
                                                                                             <double*>(rwork_density_n.data),
                                                                                             <double*>(g.data),
                                                                                             <double*>(x.data),
                                                                                             <double*>(psiw.data),
                                                                                             <double*>(psic.data),
                                                                                             <double*>(sw.data),
                                                                                             <double*>(mw.data),
                                                                                             <double*>(dmw_dpsiw.data),
                                                                                             <double*>(dmw_dpsic.data),
                                                                                             <double*>(mn.data),
                                                                                             <double*>(dmn_dpsiw.data),
                                                                                             <double*>(dmn_dpsic.data),
                                                                                             <double*>(phi_psiw.data),
                                                                                             <double*>(dphi_psiw_dpsiw.data),
                                                                                             <double*>(phi_psin.data),
                                                                                             <double*>(dphi_psin_dpsiw.data),
                                                                                             <double*>(dphi_psin_dpsic.data),
                                                                                             <double*>(aw.data),
                                                                                             <double*>(daw_dpsiw.data),
                                                                                             <double*>(daw_dpsic.data),
                                                                                             <double*>(an.data),
                                                                                             <double*>(dan_dpsiw.data),
                                                                                             <double*>(dan_dpsic.data))
        elif pskModelFlag == 1:
            tpdc.twophaseDarcy_fc_pp_sd_het_matType[VGM,ExponentialDensity,IdealGasDensity](nSimplex,
                                                                                       nPointsPerSimplex,
                                                                                       g.shape[0],
                                                                                       4,
                                                                                       <int*>(rowptr.data),
                                                                                       <int*>(colind.data),
                                                                                       <int*>(matType.data),
                                                                                       muw,
                                                                                       mun,
                                                                                       <double*>(omega.data),
                                                                                       <double*>(Kbar.data),
                                                                                       b,
                                                                                       <double*>(rwork_psk.data),
                                                                                       <int*>(iwork_psk.data),
                                                                                       <double*>(rwork_psk_tol.data),
                                                                                       <double*>(rwork_density_w.data),
                                                                                       <double*>(rwork_density_n.data),
                                                                                       <double*>(g.data),
                                                                                       <double*>(x.data),
                                                                                       <double*>(psiw.data),
                                                                                       <double*>(psic.data),
                                                                                       <double*>(sw.data),
                                                                                       <double*>(mw.data),
                                                                                       <double*>(dmw_dpsiw.data),
                                                                                       <double*>(dmw_dpsic.data),
                                                                                       <double*>(mn.data),
                                                                                       <double*>(dmn_dpsiw.data),
                                                                                       <double*>(dmn_dpsic.data),
                                                                                       <double*>(phi_psiw.data),
                                                                                       <double*>(dphi_psiw_dpsiw.data),
                                                                                       <double*>(phi_psin.data),
                                                                                       <double*>(dphi_psin_dpsiw.data),
                                                                                       <double*>(dphi_psin_dpsic.data),
                                                                                       <double*>(aw.data),
                                                                                       <double*>(daw_dpsiw.data),
                                                                                       <double*>(daw_dpsic.data),
                                                                                       <double*>(an.data),
                                                                                       <double*>(dan_dpsiw.data),
                                                                                       <double*>(dan_dpsic.data))
        else:
            tpdc.twophaseDarcy_fc_pp_sd_het_matType[SimplePSK,ExponentialDensity,IdealGasDensity](nSimplex,
                                                                                               nPointsPerSimplex,
                                                                                             g.shape[0],
                                                                                             2,
                                                                                             <int*>(rowptr.data),
                                                                                             <int*>(colind.data),
                                                                                             <int*>(matType.data),
                                                                                             muw,
                                                                                             mun,
                                                                                             <double*>(omega.data),
                                                                                             <double*>(Kbar.data),
                                                                                             b,
                                                                                             <double*>(rwork_psk.data),
                                                                                             <int*>(iwork_psk.data),
                                                                                             <double*>(rwork_psk_tol.data),
                                                                                             <double*>(rwork_density_w.data),
                                                                                             <double*>(rwork_density_n.data),
                                                                                             <double*>(g.data),
                                                                                             <double*>(x.data),
                                                                                             <double*>(psiw.data),
                                                                                             <double*>(psic.data),
                                                                                             <double*>(sw.data),
                                                                                             <double*>(mw.data),
                                                                                             <double*>(dmw_dpsiw.data),
                                                                                             <double*>(dmw_dpsic.data),
                                                                                             <double*>(mn.data),
                                                                                             <double*>(dmn_dpsiw.data),
                                                                                             <double*>(dmn_dpsic.data),
                                                                                             <double*>(phi_psiw.data),
                                                                                             <double*>(dphi_psiw_dpsiw.data),
                                                                                             <double*>(phi_psin.data),
                                                                                             <double*>(dphi_psin_dpsiw.data),
                                                                                             <double*>(dphi_psin_dpsic.data),
                                                                                             <double*>(aw.data),
                                                                                             <double*>(daw_dpsiw.data),
                                                                                             <double*>(daw_dpsic.data),
                                                                                             <double*>(an.data),
                                                                                             <double*>(dan_dpsiw.data),
                                                                                             <double*>(dan_dpsic.data))
    else:
        assert False, "configuration density_w_flag= {0} and density_n_flag= {1} not found ".format(density_w_flag, density_n_flag)

def twophaseDarcy_incompressible_split_sd_pressure_het_matType(int pskModelFlag,
                                                               np.ndarray rowptr,
                                                               np.ndarray colind,
                                                               np.ndarray matType,
                                                               double muw,
                                                               double mun,
                                                               np.ndarray omega,
                                                               np.ndarray Kbar,
                                                               double b,
                                                               np.ndarray capillaryDiffusionScaling,
                                                               np.ndarray rwork_psk,
                                                               np.ndarray iwork_psk,
                                                               np.ndarray rwork_psk_tol,
                                                               np.ndarray rwork_density_w,
                                                               np.ndarray rwork_density_n,
                                                               np.ndarray g,
                                                               np.ndarray sw,
                                                               np.ndarray grad_psic,
                                                               np.ndarray f,
                                                               np.ndarray a):
    """Evaluate the pressure coefficients

Use the split fractional flow formulation of incompressible, two-phase
Darcy flow for a heterogeneous medium, sparse diffusion rep het

    """
    cdef int nPointsPerSimplex=sw.shape[sw.ndim-1]
    cdef int nSimplex=sw.size//nPointsPerSimplex
    if pskModelFlag == 0:
        tpdc.twophaseDarcy_incompressible_split_sd_pressure_het_matType[SimplePSK](nSimplex,
                                                                              nPointsPerSimplex,
                                                                              g.shape[0],
                                                                              2,
                                                                              <int*>(rowptr.data),
                                                                              <int*>(colind.data),
                                                                              <int*>(matType.data),
                                                                              muw,
                                                                              mun,
                                                                              <double*>(omega.data),
                                                                              <double*>(Kbar.data),
                                                                              b,
                                                                              capillaryDiffusionScaling,
                                                                              <double*>(rwork_psk.data),
                                                                              <int*>(iwork_psk.data),
                                                                              <double*>(rwork_psk_tol.data),
                                                                              <double*>(rwork_density_w.data),
                                                                              <double*>(rwork_density_n.data),
                                                                              <double*>(g.data), <double*>(sw.data),
                                                                              <double*>(grad_psic.data),
                                                                              <double*>(f.data),
                                                                              <double*>(a.data))  
    elif pskModelFlag == 1:
        tpdc.twophaseDarcy_incompressible_split_sd_pressure_het_matType[VGM](nSimplex,
                                                                        nPointsPerSimplex,
                                                                        g.shape[0],
                                                                        4,
                                                                        <int*>(rowptr.data),
                                                                        <int*>(colind.data),
                                                                        <int*>(matType.data),
                                                                        muw,
                                                                        mun,
                                                                        <double*>(omega.data),
                                                                        <double*>(Kbar.data),
                                                                        b,
                                                                        capillaryDiffusionScaling,
                                                                        <double*>(rwork_psk.data),
                                                                        <int*>(iwork_psk.data),
                                                                        <double*>(rwork_psk_tol.data),
                                                                        <double*>(rwork_density_w.data),
                                                                        <double*>(rwork_density_n.data),
                                                                        <double*>(g.data),
                                                                        <double*>(sw.data),
                                                                        <double*>(grad_psic.data),
                                                                        <double*>(f.data),
                                                                        <double*>(a.data))
    else:
        tpdc.twophaseDarcy_incompressible_split_sd_pressure_het_matType[SimplePSK](nSimplex,
                                                                              nPointsPerSimplex,
                                                                              g.shape[0],
                                                                              2,
                                                                              <int*>(rowptr.data),
                                                                              <int*>(colind.data),
                                                                              <int*>(matType.data),
                                                                              muw,
                                                                              mun,
                                                                              <double*>(omega.data),
                                                                              <double*>(Kbar.data),
                                                                              b,
                                                                              capillaryDiffusionScaling,
                                                                              <double*>(rwork_psk.data),
                                                                              <int*>(iwork_psk.data),
                                                                              <double*>(rwork_psk_tol.data),
                                                                              <double*>(rwork_density_w.data),
                                                                              <double*>(rwork_density_n.data),
                                                                              <double*>(g.data),
                                                                              <double*>(sw.data),
                                                                              <double*>(grad_psic.data),
                                                                              <double*>(f.data),
                                                                              <double*>(a.data))

def twophaseDarcy_incompressible_split_sd_saturation_het_matType(int pskModelFlag,
                                                                 int density_w_flag,
                                                                 int density_n_flag,
                                                                 np.ndarray rowptr,
                                                                 np.ndarray colind,
                                                                 np.ndarray matType,
                                                                 double muw,
                                                                 double mun,
                                                                 np.ndarray omega,
                                                                 np.ndarray Kbar,
                                                                 double b,
                                                                 double capillaryDiffusionScaling,
                                                                 double advectionScaling,
                                                                 np.ndarray rwork_psk,
                                                                 np.ndarray iwork_psk,
                                                                 np.ndarray rwork_psk_tol,
                                                                 np.ndarray rwork_density_w,
                                                                 np.ndarray rwork_density_n,
                                                                 np.ndarray g,
                                                                 np.ndarray qt,
                                                                 np.ndarray psiw,
                                                                 np.ndarray sw,
                                                                 np.ndarray m,
                                                                 np.ndarray dm,
                                                                 np.ndarray phi,
                                                                 np.ndarray dphi,
                                                                 np.ndarray f,
                                                                 np.ndarray df,
                                                                 np.ndarray a,
                                                                 np.ndarray da):
    """Evaluate the saturation coefficients

Use the split fractional flow formulation of incompressible, two-phase
Darcy flow for a heterogeneous medium, sparse diffusion rep het
    """
    cdef int nPointsPerSimplex=sw.shape[sw.ndim-1]
    cdef int nSimplex=sw.size//nPointsPerSimplex
    #while debugging allow only one option
    #exponential for wetting phase
    assert density_w_flag == 1
    #for slight compressible, non-wetting phase density psk doesn't matter
    if pskModelFlag == 0:
        tpdc.twophaseDarcy_slightCompressible_split_sd_saturation_het_matType[SimplePSK,ExponentialDensity](nSimplex,
                                                                                                       nPointsPerSimplex,
                                                                                                       g.shape[0],
                                                                                                       2,
                                                                                                       <int*>(rowptr.data),
                                                                                                       <int*>(colind.data),
                                                                                                       <int*>(matType.data),
                                                                                                       muw,
                                                                                                       mun,
                                                                                                       <double*>(omega.data),
                                                                                                       <double*>(Kbar.data),
                                                                                                       b,
                                                                                                       capillaryDiffusionScaling,
                                                                                                       advectionScaling,
                                                                                                       <double*>(rwork_psk.data),
                                                                                                       <int*>(iwork_psk.data),
                                                                                                       <double*>(rwork_psk_tol.data),
                                                                                                       <double*>(rwork_density_w.data),
                                                                                                       <double*>(rwork_density_n.data),
                                                                                                       <double*>(g.data), <double*>(qt.data),
                                                                                                       <double*>(psiw.data),
                                                                                                       <double*>(sw.data), 
                                                                                                       <double*>(m.data),
                                                                                                       <double*>(dm.data),
                                                                                                       <double*>(phi.data),
                                                                                                       <double*>(dphi.data),
                                                                                                       <double*>(f.data),
                                                                                                       <double*>(df.data),
                                                                                                       <double*>(a.data),
                                                                                                       <double*>(da.data))        
    elif pskModelFlag == 1:
        tpdc.twophaseDarcy_slightCompressible_split_sd_saturation_het_matType[VGM,ExponentialDensity](nSimplex,
                                                                                                 nPointsPerSimplex,
                                                                                                 g.shape[0],
                                                                                                 4,
                                                                                                 <int*>(rowptr.data),
                                                                                                 <int*>(colind.data),
                                                                                                 <int*>(matType.data),
                                                                                                 muw,
                                                                                                 mun,
                                                                                                 <double*>(omega.data),
                                                                                                 <double*>(Kbar.data),
                                                                                                 b,
                                                                                                 capillaryDiffusionScaling,
                                                                                                 advectionScaling,
                                                                                                 <double*>(rwork_psk.data),
                                                                                                 <int*>(iwork_psk.data),
                                                                                                 <double*>(rwork_psk_tol.data),
                                                                                                 <double*>(rwork_density_w.data),
                                                                                                 <double*>(rwork_density_n.data),
                                                                                                 <double*>(g.data),
                                                                                                 <double*>(qt.data),
                                                                                                 <double*>(psiw.data),
                                                                                                 <double*>(sw.data), 
                                                                                                 <double*>(m.data),
                                                                                                 <double*>(dm.data),
                                                                                                 <double*>(phi.data),
                                                                                                 <double*>(dphi.data),
                                                                                                 <double*>(f.data),
                                                                                                 <double*>(df.data),
                                                                                                 <double*>(a.data),
                                                                                                 <double*>(da.data))
    else:
        tpdc.twophaseDarcy_slightCompressible_split_sd_saturation_het_matType[VGM,ExponentialDensity](nSimplex,
                                                                                                 nPointsPerSimplex,
                                                                                                 g.shape[0],
                                                                                                 4,
                                                                                                 <int*>(rowptr.data),
                                                                                                 <int*>(colind.data),
                                                                                                 <int*>(matType.data),
                                                                                                 muw,
                                                                                                 mun,
                                                                                                 <double*>(omega.data),
                                                                                                 <double*>(Kbar.data),
                                                                                                 b,
                                                                                                 capillaryDiffusionScaling,
                                                                                                 advectionScaling,
                                                                                                 <double*>(rwork_psk.data),
                                                                                                 <int*>(iwork_psk.data),
                                                                                                 <double*>(rwork_psk_tol.data),
                                                                                                 <double*>(rwork_density_w.data),
                                                                                                 <double*>(rwork_density_n.data),
                                                                                                 <double*>(g.data),
                                                                                                 <double*>(qt.data),
                                                                                                 <double*>(psiw.data),
                                                                                                 <double*>(sw.data), 
                                                                                                 <double*>(m.data),
                                                                                                 <double*>(dm.data),
                                                                                                 <double*>(phi.data),
                                                                                                 <double*>(dphi.data),
                                                                                                 <double*>(f.data),
                                                                                                 <double*>(df.data),
                                                                                                 <double*>(a.data),
                                                                                                 <double*>(da.data))

def twophaseDarcy_slightCompressible_split_sd_pressure_het_matType(int pskModelFlag,
                                                                   int density_w_flag,
                                                                   int density_n_flag,
                                                                   np.ndarray rowptr,
                                                                   np.ndarray colind,
                                                                   np.ndarray matType,
                                                                   double muw,
                                                                   double mun,
                                                                   np.ndarray omega,
                                                                   np.ndarray Kbar,
                                                                   double b,
                                                                   double capillaryDiffusionScaling,
                                                                   double advectionScaling,
                                                                   np.ndarray rwork_psk,
                                                                   np.ndarray iwork_psk,
                                                                   np.ndarray rwork_psk_tol,
                                                                   np.ndarray rwork_density_w,
                                                                   np.ndarray rwork_density_n,
                                                                   np.ndarray g,
                                                                   np.ndarray qt,
                                                                   np.ndarray psiw,
                                                                   np.ndarray sw,
                                                                   np.ndarray m,
                                                                   np.ndarray dm,
                                                                   np.ndarray phi,
                                                                   np.ndarray dphi,
                                                                   np.ndarray f,
                                                                   np.ndarray df,
                                                                   np.ndarray a,
                                                                   np.ndarray da):
    """Evaluate the pressure coefficients

Use the split fractional flow formulation of slight compressible,
two-phase Darcy flow for a heterogeneous medium, sparse diffusion rep
het

    """
    cdef int nPointsPerSimplex=sw.shape[sw.ndim-1]
    cdef int nSimplex=sw.size//nPointsPerSimplex
    #while debugging allow only one option
    #exponential for wetting phase
    assert density_w_flag == 1
    #for slight compressible, non-wetting phase density psk doesn't matter
    if pskModelFlag == 0:
        tpdc.twophaseDarcy_slightCompressible_split_sd_saturation_het_matType[SimplePSK,ExponentialDensity](nSimplex,
                                                                                                       nPointsPerSimplex,
                                                                                                       g.shape[0],
                                                                                                       2,
                                                                                                       <int*>(rowptr.data),
                                                                                                       <int*>(colind.data),
                                                                                                       <int*>(matType.data),
                                                                                                       muw,
                                                                                                       mun,
                                                                                                       <double*>(omega.data),
                                                                                                       <double*>(Kbar.data),
                                                                                                       b,
                                                                                                       capillaryDiffusionScaling,
                                                                                                       advectionScaling,
                                                                                                       <double*>(rwork_psk.data),
                                                                                                       <int*>(iwork_psk.data),
                                                                                                       <double*>(rwork_psk_tol.data),
                                                                                                       <double*>(rwork_density_w.data),
                                                                                                       <double*>(rwork_density_n.data),
                                                                                                       <double*>(g.data),
                                                                                                       <double*>(qt.data),
                                                                                                       <double*>(psiw.data),
                                                                                                       <double*>(sw.data), 
                                                                                                       <double*>(m.data),
                                                                                                       <double*>(dm.data),
                                                                                                       <double*>(phi.data),
                                                                                                       <double*>(dphi.data), <double*>(f.data),
                                                                                                       <double*>(df.data), <double*>(a.data), <double*>(da.data))
    elif pskModelFlag == 1:
        tpdc.twophaseDarcy_slightCompressible_split_sd_saturation_het_matType[VGM,ExponentialDensity](nSimplex,
                                                                                                  nPointsPerSimplex,
                                                                                                 g.shape[0],
                                                                                                 4,
                                                                                                 <int*>(rowptr.data),
                                                                                                 <int*>(colind.data),
                                                                                                 <int*>(matType.data),
                                                                                                 muw,
                                                                                                 mun,
                                                                                                 <double*>(omega.data),
                                                                                                 <double*>(Kbar.data),
                                                                                                 b,
                                                                                                 capillaryDiffusionScaling,advectionScaling,
                                                                                                 <double*>(rwork_psk.data),<int*>(iwork_psk.data),
                                                                                                 <double*>(rwork_psk_tol.data),
                                                                                                 <double*>(rwork_density_w.data),
                                                                                                 <double*>(rwork_density_n.data),
                                                                                                 <double*>(g.data), <double*>(qt.data), <double*>(psiw.data), <double*>(sw.data), 
                                                                                                 <double*>(m.data), <double*>(dm.data), <double*>(phi.data), <double*>(dphi.data), <double*>(f.data),
                                                                                                 <double*>(df.data), <double*>(a.data), <double*>(da.data))
    else:
        tpdc.twophaseDarcy_slightCompressible_split_sd_saturation_het_matType[VGM,ExponentialDensity](nSimplex,
                                                                                                  nPointsPerSimplex,
                                                                                                 g.shape[0],
                                                                                                 4,
                                                                                                 <int*>(rowptr.data),
                                                                                                 <int*>(colind.data),
                                                                                                 <int*>(matType.data),
                                                                                                 muw,
                                                                                                 mun,
                                                                                                 <double*>(omega.data),
                                                                                                 <double*>(Kbar.data),
                                                                                                 b,
                                                                                                 capillaryDiffusionScaling,advectionScaling,
                                                                                                 <double*>(rwork_psk.data),<int*>(iwork_psk.data),<double*>(rwork_psk_tol.data),
                                                                                                 <double*>(rwork_density_w.data),
                                                                                                 <double*>(rwork_density_n.data),
                                                                                                 <double*>(g.data), <double*>(qt.data), <double*>(psiw.data), <double*>(sw.data), 
                                                                                                 <double*>(m.data), <double*>(dm.data), <double*>(phi.data), <double*>(dphi.data), <double*>(f.data),
                                                                                                 <double*>(df.data), <double*>(a.data), <double*>(da.data))

def twophaseDarcy_slightCompressible_split_sd_saturation_het_matType(                         int pskModelFlag,
                         int density_w_flag,
                         int density_n_flag,
                         np.ndarray rowptr,
                         np.ndarray colind,
                         np.ndarray matType,
                         double muw,
                         double mun,
                         np.ndarray omega,
                         np.ndarray Kbar,
                         double b,
                         double capillaryDiffusionScaling,
                         double advectionScaling,
                         np.ndarray rwork_psk,
                         np.ndarray iwork_psk,
                         np.ndarray rwork_psk_tol,
                         np.ndarray rwork_density_w,
                         np.ndarray rwork_density_n,
                         np.ndarray g, np.ndarray qt, np.ndarray psiw, np.ndarray sw, np.ndarray m, np.ndarray dm, np.ndarray phi, np.ndarray dphi,
                         np.ndarray f, np.ndarray df, np.ndarray a, np.ndarray da):
    """Evaluate the saturation coefficients

Use the split fractional flow formulation of slight compressible,
two-phase Darcy flow for a heterogeneous medium, sparse diffusion rep
het

    """
    cdef int nPointsPerSimplex=sw.shape[sw.ndim-1]
    cdef int nSimplex=sw.size//nPointsPerSimplex
    if density_n_flag == 1: #nonwetting exponential
        if pskModelFlag == 0:
            tpdc.twophaseDarcy_compressibleN_split_sd_saturation_het_matType[SimplePSK,ExponentialDensity](nSimplex,
                                                                                                      nPointsPerSimplex,
                                                                                                      g.shape[0],
                                                                                                      2,
                                                                                                      <int*>(rowptr.data),
                                                                                                      <int*>(colind.data),
                                                                                                      <int*>(matType.data),
                                                                                                      muw,
                                                                                                      mun,
                                                                                                      <double*>(omega.data),
                                                                                                      <double*>(Kbar.data),
                                                                                                      b,
                                                                                                      capillaryDiffusionScaling,advectionScaling,
                                                                                                      <double*>(rwork_psk.data),<int*>(iwork_psk.data),
                                                                                                      <double*>(rwork_psk_tol.data),
                                                                                                      <double*>(rwork_density_w.data),
                                                                                                      <double*>(rwork_density_n.data),
                                                                                                      <double*>(g.data), <double*>(qt.data), <double*>(psiw.data),<double*>(sw.data), 
                                                                                                      <double*>(m.data), <double*>(dm.data), <double*>(phi.data), <double*>(dphi.data), <double*>(f.data),
                                                                                                      <double*>(df.data), <double*>(a.data), <double*>(da.data))
        elif pskModelFlag == 1:
            tpdc.twophaseDarcy_compressibleN_split_sd_saturation_het_matType[VGM,ExponentialDensity](nSimplex,
                                                                                                nPointsPerSimplex,
                                                                                                g.shape[0],
                                                                                                4,
                                                                                                <int*>(rowptr.data),
                                                                                                <int*>(colind.data),
                                                                                                <int*>(matType.data),
                                                                                                muw,
                                                                                                mun,
                                                                                                <double*>(omega.data),
                                                                                                <double*>(Kbar.data),
                                                                                                b,
                                                                                                capillaryDiffusionScaling,advectionScaling,
                                                                                                <double*>(rwork_psk.data),<int*>(iwork_psk.data),
                                                                                                <double*>(rwork_psk_tol.data),
                                                                                                <double*>(rwork_density_w.data),
                                                                                                <double*>(rwork_density_n.data),
                                                                                                <double*>(g.data), <double*>(qt.data), <double*>(psiw.data), <double*>(sw.data), 
                                                                                                <double*>(m.data), <double*>(dm.data), <double*>(phi.data), <double*>(dphi.data), <double*>(f.data),
                                                                                                <double*>(df.data), <double*>(a.data), <double*>(da.data))
        else:
            tpdc.twophaseDarcy_compressibleN_split_sd_saturation_het_matType[VGM,ExponentialDensity](nSimplex,
                                                                                                nPointsPerSimplex,
                                                                                                g.shape[0],
                                                                                                4,
                                                                                                <int*>(rowptr.data),
                                                                                                <int*>(colind.data),
                                                                                                <int*>(matType.data),
                                                                                                muw,
                                                                                                mun,
                                                                                                <double*>(omega.data),
                                                                                                <double*>(Kbar.data),
                                                                                                b,
                                                                                                capillaryDiffusionScaling,advectionScaling,
                                                                                                <double*>(rwork_psk.data),
                                                                                                <int*>(iwork_psk.data),
                                                                                                <double*>(rwork_psk_tol.data),
                                                                                                <double*>(rwork_density_w.data),
                                                                                                <double*>(rwork_density_n.data),
                                                                                                <double*>(g.data), <double*>(qt.data), <double*>(psiw.data), <double*>(sw.data), 
                                                                                                <double*>(m.data), <double*>(dm.data), <double*>(phi.data), <double*>(dphi.data), <double*>(f.data),
                                                                                                <double*>(df.data), <double*>(a.data), <double*>(da.data))
    elif density_n_flag == 2:#nonwetting IdealGas
        if pskModelFlag == 0:
            tpdc.twophaseDarcy_compressibleN_split_sd_saturation_het_matType[SimplePSK,IdealGasDensity](nSimplex,
                                                                                                   nPointsPerSimplex,
                                                                                                   g.shape[0],
                                                                                                   2,
                                                                                                   <int*>(rowptr.data),
                                                                                                   <int*>(colind.data),
                                                                                                   <int*>(matType.data),
                                                                                                   muw,
                                                                                                   mun,
                                                                                                   <double*>(omega.data),
                                                                                                   <double*>(Kbar.data),
                                                                                                   b,
                                                                                                   capillaryDiffusionScaling,advectionScaling,
                                                                                                   <double*>(rwork_psk.data),<int*>(iwork_psk.data),
                                                                                                   <double*>(rwork_psk_tol.data),
                                                                                                   <double*>(rwork_density_w.data),
                                                                                                   <double*>(rwork_density_n.data),
                                                                                                   <double*>(g.data), <double*>(qt.data), <double*>(psiw.data),<double*>(sw.data), 
                                                                                                   <double*>(m.data), <double*>(dm.data), <double*>(phi.data), <double*>(dphi.data), <double*>(f.data),
                                                                                                   <double*>(df.data), <double*>(a.data), <double*>(da.data))
        elif pskModelFlag == 1:
            tpdc.twophaseDarcy_compressibleN_split_sd_saturation_het_matType[VGM,IdealGasDensity](nSimplex,
                                                                                             nPointsPerSimplex,
                                                                                             g.shape[0],
                                                                                             4,
                                                                                             <int*>(rowptr.data),
                                                                                             <int*>(colind.data),
                                                                                             <int*>(matType.data),
                                                                                             muw,
                                                                                             mun,
                                                                                             <double*>(omega.data),
                                                                                             <double*>(Kbar.data),
                                                                                             b,
                                                                                             capillaryDiffusionScaling,advectionScaling,
                                                                                             <double*>(rwork_psk.data),<int*>(iwork_psk.data),
                                                                                             <double*>(rwork_psk_tol.data),
                                                                                             <double*>(rwork_density_w.data),
                                                                                             <double*>(rwork_density_n.data),
                                                                                             <double*>(g.data), <double*>(qt.data), <double*>(psiw.data), <double*>(sw.data), 
                                                                                             <double*>(m.data), <double*>(dm.data), <double*>(phi.data), <double*>(dphi.data), <double*>(f.data),
                                                                                             <double*>(df.data), <double*>(a.data), <double*>(da.data))
        else:
            tpdc.twophaseDarcy_compressibleN_split_sd_saturation_het_matType[VGM,IdealGasDensity](nSimplex,
                                                                                             nPointsPerSimplex,
                                                                                             g.shape[0],
                                                                                             4,
                                                                                             <int*>(rowptr.data),
                                                                                             <int*>(colind.data),
                                                                                             <int*>(matType.data),
                                                                                             muw,
                                                                                             mun,
                                                                                             <double*>(omega.data),
                                                                                             <double*>(Kbar.data),
                                                                                             b,
                                                                                             capillaryDiffusionScaling,advectionScaling,
                                                                                             <double*>(rwork_psk.data),<int*>(iwork_psk.data),
                                                                                             <double*>(rwork_psk_tol.data),
                                                                                             <double*>(rwork_density_w.data),
                                                                                             <double*>(rwork_density_n.data),
                                                                                             <double*>(g.data), <double*>(qt.data), <double*>(psiw.data), <double*>(sw.data), 
                                                                                             <double*>(m.data), <double*>(dm.data), <double*>(phi.data), <double*>(dphi.data), <double*>(f.data),
                                                                                             <double*>(df.data), <double*>(a.data), <double*>(da.data))
    else:
        assert False, "configuration density_w_flag= {0} and density_n_flag= {1} not found ".format(density_w_flag, density_n_flag)

def twophaseDarcy_compressibleN_split_sd_pressure_het_matType(                         int pskModelFlag,
                         int density_w_flag,
                         int density_n_flag,
                         np.ndarray rowptr,
                         np.ndarray colind,
                         np.ndarray matType,
                         double muw,
                         double mun,
                         np.ndarray omega,
                         np.ndarray Kbar,
                         double b,
                         double capillaryDiffusionScaling,
                         np.ndarray rwork_psk,
                         np.ndarray iwork_psk,
                         np.ndarray rwork_psk_tol,
                         np.ndarray rwork_density_w,
                         np.ndarray rwork_density_n,
                         np.ndarray g, np.ndarray sw, np.ndarray psiw, np.ndarray psin, np.ndarray grad_psic, np.ndarray m, np.ndarray dm, np.ndarray f, np.ndarray a):
    """Evaluate the pressure coefficients

Use the split fractional flow formulation of compressible non-wetting
phase, two-phase Darcy flow for a heterogeneous medium, sparse
diffusion rep het

    """
    cdef int nPointsPerSimplex=sw.shape[sw.ndim-1]
    cdef int nSimplex=sw.size//nPointsPerSimplex
    if density_n_flag == 1: #nonwetting exponential
        if pskModelFlag == 0:
            tpdc.twophaseDarcy_compressibleN_split_sd_pressure_het_matType[SimplePSK,ExponentialDensity](nSimplex,
                                                                                                    nPointsPerSimplex,
                                                                                                    g.shape[0],
                                                                                                    2,
                                                                                                    <int*>(rowptr.data),
                                                                                                    <int*>(colind.data),
                                                                                                    <int*>(matType.data),
                                                                                                    muw,
                                                                                                    mun,
                                                                                                    <double*>(omega.data),
                                                                                                    <double*>(Kbar.data),
                                                                                                    b,
                                                                                                    capillaryDiffusionScaling,
                                                                                                    <double*>(rwork_psk.data),<int*>(iwork_psk.data),
                                                                                                    <double*>(rwork_psk_tol.data),
                                                                                                    <double*>(rwork_density_w.data),
                                                                                                    <double*>(rwork_density_n.data),
                                                                                                    <double*>(g.data), <double*>(sw.data), <double*>(psiw.data), <double*>(psin.data), <double*>(grad_psic.data), 
                                                                                                    <double*>(m.data), <double*>(dm.data), <double*>(f.data), <double*>(a.data))
        elif pskModelFlag == 1:
            tpdc.twophaseDarcy_compressibleN_split_sd_pressure_het_matType[VGM,ExponentialDensity](nSimplex,
                                                                                              nPointsPerSimplex,
                                                                                              g.shape[0],
                                                                                              4,
                                                                                              <int*>(rowptr.data),
                                                                                              <int*>(colind.data),
                                                                                              <int*>(matType.data),
                                                                                              muw,
                                                                                              mun,
                                                                                              <double*>(omega.data),
                                                                                              <double*>(Kbar.data),
                                                                                              b,
                                                                                              capillaryDiffusionScaling,
                                                                                              <double*>(rwork_psk.data),<int*>(iwork_psk.data),
                                                                                              <double*>(rwork_psk_tol.data),
                                                                                              <double*>(rwork_density_w.data),
                                                                                              <double*>(rwork_density_n.data),
                                                                                              <double*>(g.data), <double*>(sw.data), <double*>(psiw.data), <double*>(psin.data), <double*>(grad_psic.data), 
                                                                                              <double*>(m.data), <double*>(dm.data), <double*>(f.data), <double*>(a.data))
        else:
            tpdc.twophaseDarcy_compressibleN_split_sd_pressure_het_matType[VGM,ExponentialDensity](nSimplex,
                                                                                              nPointsPerSimplex,
                                                                                              g.shape[0],
                                                                                              4,
                                                                                              <int*>(rowptr.data),
                                                                                              <int*>(colind.data),
                                                                                              <int*>(matType.data),
                                                                                              muw,
                                                                                              mun,
                                                                                              <double*>(omega.data),
                                                                                              <double*>(Kbar.data),
                                                                                              b,
                                                                                              capillaryDiffusionScaling,
                                                                                              <double*>(rwork_psk.data),
                                                                                              <int*>(iwork_psk.data),
                                                                                              <double*>(rwork_psk_tol.data),
                                                                                              <double*>(rwork_density_w.data),
                                                                                              <double*>(rwork_density_n.data),
                                                                                              <double*>(g.data), <double*>(sw.data), <double*>(psiw.data), <double*>(psin.data), <double*>(grad_psic.data), 
                                                                                              <double*>(m.data), <double*>(dm.data), <double*>(f.data), <double*>(a.data))
    elif density_n_flag == 2: #nonwetting IdealGas
        if pskModelFlag == 0:
            tpdc.twophaseDarcy_compressibleN_split_sd_pressure_het_matType[SimplePSK,IdealGasDensity](nSimplex,
                                                                                                 nPointsPerSimplex,
                                                                                                 g.shape[0],
                                                                                                 2,
                                                                                                 <int*>(rowptr.data),
                                                                                                 <int*>(colind.data),
                                                                                                 <int*>(matType.data),
                                                                                                 muw,
                                                                                                 mun,
                                                                                                 <double*>(omega.data),
                                                                                                 <double*>(Kbar.data),
                                                                                                 b,
                                                                                                 capillaryDiffusionScaling,
                                                                                                 <double*>(rwork_psk.data),
                                                                                                 <int*>(iwork_psk.data),
                                                                                                 <double*>(rwork_psk_tol.data),
                                                                                                 <double*>(rwork_density_w.data),
                                                                                                 <double*>(rwork_density_n.data),
                                                                                                 <double*>(g.data), <double*>(sw.data), <double*>(psiw.data), <double*>(psin.data), <double*>(grad_psic.data), 
                                                                                                 <double*>(m.data), <double*>(dm.data), <double*>(f.data), <double*>(a.data))
        elif pskModelFlag == 1:
            tpdc.twophaseDarcy_compressibleN_split_sd_pressure_het_matType[VGM,IdealGasDensity](nSimplex,
                                                                                           nPointsPerSimplex,
                                                                                           g.shape[0],
                                                                                           4,
                                                                                           <int*>(rowptr.data),
                                                                                           <int*>(colind.data),
                                                                                           <int*>(matType.data),
                                                                                           muw,
                                                                                           mun,
                                                                                           <double*>(omega.data),
                                                                                           <double*>(Kbar.data),
                                                                                           b,
                                                                                           capillaryDiffusionScaling,
                                                                                           <double*>(rwork_psk.data),
                                                                                           <int*>(iwork_psk.data),
                                                                                           <double*>(rwork_psk_tol.data),
                                                                                           <double*>(rwork_density_w.data),
                                                                                           <double*>(rwork_density_n.data),
                                                                                           <double*>(g.data), <double*>(sw.data), <double*>(psiw.data), <double*>(psin.data), <double*>(grad_psic.data), 
                                                                                           <double*>(m.data), <double*>(dm.data), <double*>(f.data), <double*>(a.data))
        else:
            tpdc.twophaseDarcy_compressibleN_split_sd_pressure_het_matType[VGM,IdealGasDensity](nSimplex,
                                                                                           nPointsPerSimplex,
                                                                                           g.shape[0],
                                                                                           4,
                                                                                           <int*>(rowptr.data),
                                                                                           <int*>(colind.data),
                                                                                           <int*>(matType.data),
                                                                                           muw,
                                                                                           mun,
                                                                                           <double*>(omega.data),
                                                                                           <double*>(Kbar.data),
                                                                                           b,
                                                                                           capillaryDiffusionScaling,
                                                                                           <double*>(rwork_psk.data),
                                                                                           <int*>(iwork_psk.data),
                                                                                           <double*>(rwork_psk_tol.data),
                                                                                           <double*>(rwork_density_w.data),
                                                                                           <double*>(rwork_density_n.data),
                                                                                           <double*>(g.data), <double*>(sw.data), <double*>(psiw.data), <double*>(psin.data), <double*>(grad_psic.data), 
                                                                                           <double*>(m.data), <double*>(dm.data), <double*>(f.data), <double*>(a.data))
    else:
        assert False, "configuration density_w_flag= {0} and density_n_flag= {1} not found ".format(density_w_flag, density_n_flag)

def twophaseDarcy_compressibleN_split_sd_saturation_het_matType(                         int pskModelFlag,
                         int density_w_flag,
                         int density_n_flag,
                         np.ndarray rowptr,
                         np.ndarray colind,
                         np.ndarray matType,
                         double muw,
                         double mun,
                         np.ndarray omega,
                         np.ndarray Kbar,
                         double b,
                         double capillaryDiffusionScaling,
                         double advectionScaling,
                         np.ndarray rwork_psk,
                         np.ndarray iwork_psk,
                         np.ndarray rwork_psk_tol,
                         np.ndarray rwork_density_w,
                         np.ndarray rwork_density_n,
                         np.ndarray g, np.ndarray qt, np.ndarray psiw, np.ndarray sw, np.ndarray m, np.ndarray dm, np.ndarray phi, np.ndarray dphi,
                         np.ndarray f, np.ndarray df, np.ndarray a, np.ndarray da):
    """Evaluate the saturation coefficients

Use the split fractional flow formulation of compressible non-wetting
phase, two-phase Darcy flow for a heterogeneous medium, sparse
diffusion rep het

    """
    cdef int nPointsPerSimplex=sw.shape[sw.ndim-1]
    cdef int nSimplex=sw.size//nPointsPerSimplex
    if density_n_flag == 1: #nonwetting exponential
        if pskModelFlag == 0:
            tpdc.twophaseDarcy_compressibleN_split_sd_saturation_het_matType[SimplePSK,ExponentialDensity](nSimplex,
                                                                                                      nPointsPerSimplex,
                                                                                                      g.shape[0],
                                                                                                      2,
                                                                                                      <int*>(rowptr.data),
                                                                                                      <int*>(colind.data),
                                                                                                      <int*>(matType.data),
                                                                                                      muw,
                                                                                                      mun,
                                                                                                      <double*>(omega.data),
                                                                                                      <double*>(Kbar.data),
                                                                                                      b,
                                                                                                      capillaryDiffusionScaling,advectionScaling,
                                                                                                      <double*>(rwork_psk.data),<int*>(iwork_psk.data),
                                                                                                      <double*>(rwork_psk_tol.data),
                                                                                                      <double*>(rwork_density_w.data),
                                                                                                      <double*>(rwork_density_n.data),
                                                                                                      <double*>(g.data), <double*>(qt.data), <double*>(psiw.data),<double*>(sw.data), 
                                                                                                      <double*>(m.data), <double*>(dm.data), <double*>(phi.data), <double*>(dphi.data), <double*>(f.data),
                                                                                                      <double*>(df.data), <double*>(a.data), <double*>(da.data))
        elif pskModelFlag == 1:
            tpdc.twophaseDarcy_compressibleN_split_sd_saturation_het_matType[VGM,ExponentialDensity](nSimplex,
                                                                                                nPointsPerSimplex,
                                                                                                g.shape[0],
                                                                                                4,
                                                                                                <int*>(rowptr.data),
                                                                                                <int*>(colind.data),
                                                                                                <int*>(matType.data),
                                                                                                muw,
                                                                                                mun,
                                                                                                <double*>(omega.data),
                                                                                                <double*>(Kbar.data),
                                                                                                b,
                                                                                                capillaryDiffusionScaling,advectionScaling,
                                                                                                <double*>(rwork_psk.data),<int*>(iwork_psk.data),
                                                                                                <double*>(rwork_psk_tol.data),
                                                                                                <double*>(rwork_density_w.data),
                                                                                                <double*>(rwork_density_n.data),
                                                                                                <double*>(g.data), <double*>(qt.data), <double*>(psiw.data), <double*>(sw.data), 
                                                                                                <double*>(m.data), <double*>(dm.data), <double*>(phi.data), <double*>(dphi.data), <double*>(f.data),
                                                                                                <double*>(df.data), <double*>(a.data), <double*>(da.data))
        else:
            tpdc.twophaseDarcy_compressibleN_split_sd_saturation_het_matType[VGM,ExponentialDensity](nSimplex,
                                                                                                nPointsPerSimplex,
                                                                                                g.shape[0],
                                                                                                4,
                                                                                                <int*>(rowptr.data),
                                                                                                <int*>(colind.data),
                                                                                                <int*>(matType.data),
                                                                                                muw,
                                                                                                mun,
                                                                                                <double*>(omega.data),
                                                                                                <double*>(Kbar.data),
                                                                                                b,
                                                                                                capillaryDiffusionScaling,advectionScaling,
                                                                                                <double*>(rwork_psk.data),
                                                                                                <int*>(iwork_psk.data),
                                                                                                <double*>(rwork_psk_tol.data),
                                                                                                <double*>(rwork_density_w.data),
                                                                                                <double*>(rwork_density_n.data),
                                                                                                <double*>(g.data), <double*>(qt.data), <double*>(psiw.data), <double*>(sw.data), 
                                                                                                <double*>(m.data), <double*>(dm.data), <double*>(phi.data), <double*>(dphi.data), <double*>(f.data),
                                                                                                <double*>(df.data), <double*>(a.data), <double*>(da.data))
    elif density_n_flag == 2:#nonwetting IdealGas
        if pskModelFlag == 0:
            tpdc.twophaseDarcy_compressibleN_split_sd_saturation_het_matType[SimplePSK,IdealGasDensity](nSimplex,
                                                                                                   nPointsPerSimplex,
                                                                                                   g.shape[0],
                                                                                                   2,
                                                                                                   <int*>(rowptr.data),
                                                                                                   <int*>(colind.data),
                                                                                                   <int*>(matType.data),
                                                                                                   muw,
                                                                                                   mun,
                                                                                                   <double*>(omega.data),
                                                                                                   <double*>(Kbar.data),
                                                                                                   b,
                                                                                                   capillaryDiffusionScaling,advectionScaling,
                                                                                                   <double*>(rwork_psk.data),<int*>(iwork_psk.data),
                                                                                                   <double*>(rwork_psk_tol.data),
                                                                                                   <double*>(rwork_density_w.data),
                                                                                                   <double*>(rwork_density_n.data),
                                                                                                   <double*>(g.data), <double*>(qt.data), <double*>(psiw.data),<double*>(sw.data), 
                                                                                                   <double*>(m.data), <double*>(dm.data), <double*>(phi.data), <double*>(dphi.data), <double*>(f.data),
                                                                                                   <double*>(df.data), <double*>(a.data), <double*>(da.data))
        elif pskModelFlag == 1:
            tpdc.twophaseDarcy_compressibleN_split_sd_saturation_het_matType[VGM,IdealGasDensity](nSimplex,
                                                                                             nPointsPerSimplex,
                                                                                             g.shape[0],
                                                                                             4,
                                                                                             <int*>(rowptr.data),
                                                                                             <int*>(colind.data),
                                                                                             <int*>(matType.data),
                                                                                             muw,
                                                                                             mun,
                                                                                             <double*>(omega.data),
                                                                                             <double*>(Kbar.data),
                                                                                             b,
                                                                                             capillaryDiffusionScaling,advectionScaling,
                                                                                             <double*>(rwork_psk.data),<int*>(iwork_psk.data),
                                                                                             <double*>(rwork_psk_tol.data),
                                                                                             <double*>(rwork_density_w.data),
                                                                                             <double*>(rwork_density_n.data),
                                                                                             <double*>(g.data), <double*>(qt.data), <double*>(psiw.data), <double*>(sw.data), 
                                                                                             <double*>(m.data), <double*>(dm.data), <double*>(phi.data), <double*>(dphi.data), <double*>(f.data),
                                                                                             <double*>(df.data), <double*>(a.data), <double*>(da.data))
        else:
            tpdc.twophaseDarcy_compressibleN_split_sd_saturation_het_matType[VGM,IdealGasDensity](nSimplex,
                                                                                             nPointsPerSimplex,
                                                                                             g.shape[0],
                                                                                             4,
                                                                                             <int*>(rowptr.data),
                                                                                             <int*>(colind.data),
                                                                                             <int*>(matType.data),
                                                                                             muw,
                                                                                             mun,
                                                                                             <double*>(omega.data),
                                                                                             <double*>(Kbar.data),
                                                                                             b,
                                                                                             capillaryDiffusionScaling,advectionScaling,
                                                                                             <double*>(rwork_psk.data),<int*>(iwork_psk.data),
                                                                                             <double*>(rwork_psk_tol.data),
                                                                                             <double*>(rwork_density_w.data),
                                                                                             <double*>(rwork_density_n.data),
                                                                                             <double*>(g.data), <double*>(qt.data), <double*>(psiw.data), <double*>(sw.data), 
                                                                                             <double*>(m.data), <double*>(dm.data), <double*>(phi.data), <double*>(dphi.data), <double*>(f.data),
                                                                                             <double*>(df.data), <double*>(a.data), <double*>(da.data))
    else:
        assert False, "configuration density_w_flag= {0} and density_n_flag= {1} not found ".format(density_w_flag, density_n_flag)

def twophaseDarcy_vol_frac(                      np.ndarray matType,
                         np.ndarray omega,
                         np.ndarray sw,
                         np.ndarray vol_frac_w,
                         np.ndarray vol_frac_n):
    """compute phase volume fractions

Use point-vals for sw and zoned material types for porosity (omega)

    """
    cdef int nPointsPerSimplex=sw.shape[sw.ndim-1]
    cdef int nSimplex=sw.size//nPointsPerSimplex
    tpdc.twophaseDarcy_vol_frac(nSimplex,
                           nPointsPerSimplex,
                           <int*>(matType.data),
                           <double*>(omega.data),
                           <double*>(sw.data),
                           <double*>(vol_frac_w.data),
                           <double*>(vol_frac_n.data))

def generateSplineTables(int pskModelFlag,
                         int startIndex, #where to insert into splineTable
                         int calcFlag,   #f(Sw) or f(psic)
                         np.ndarray domain,     #Sw or psic
                         np.ndarray rwork_psk,  #psk parameter values
                         np.ndarray iwork_psk,
                         np.ndarray rwork_psk_tol,
                         np.ndarray splineTable):
    """generate spline table look up arrays"""
    nknots = domain.shape[0]
    if pskModelFlag == 0:
        tpdc.generateSplineTables[SimplePSK](nknots,
                                        startIndex,
                                        calcFlag,
                                        <double*>(domain.data),
                                        <double*>(rwork_psk.data),
                                        <int*>(iwork_psk.data),
                                        <double*>(rwork_psk_tol.data),
                                        <double*>(splineTable.data))
    elif pskModelFlag == 1:
        tpdc.generateSplineTables[VGM](nknots,
                                  startIndex,
                                  calcFlag,
                                  <double*>(domain.data),
                                  <double*>(rwork_psk.data),
                                  <int*>(iwork_psk.data),
                                  <double*>(rwork_psk_tol.data),
                                  <double*>(splineTable.data))
    elif pskModelFlag == 2:
        tpdc.generateSplineTables[VGB](nknots,
                                  startIndex,
                                  calcFlag,
                                  <double*>(domain.data),
                                  <double*>(rwork_psk.data),
                                  <int*>(iwork_psk.data),
                                  <double*>(rwork_psk_tol.data),
                                  <double*>(splineTable.data))
    elif pskModelFlag == 3:
        tpdc.generateSplineTables[BCM](nknots,
                                  startIndex,
                                  calcFlag,
                                  <double*>(domain.data),
                                  <double*>(rwork_psk.data),
                                  <int*>(iwork_psk.data),
                                  <double*>(rwork_psk_tol.data),
                                  <double*>(splineTable.data))
    elif pskModelFlag == 4:
        tpdc.generateSplineTables[BCB](nknots,
                                  startIndex,
                                  calcFlag,
                                  <double*>(domain.data),
                                  <double*>(rwork_psk.data),
                                  <int*>(iwork_psk.data),
                                  <double*>(rwork_psk_tol.data),
                                  <double*>(splineTable.data))
    else:
        tpdc.generateSplineTables[VGM](nknots,
                                  startIndex,
                                  calcFlag,
                                  <double*>(domain.data),
                                  <double*>(rwork_psk.data),
                                  <int*>(iwork_psk.data),
                                  <double*>(rwork_psk_tol.data),
                                  <double*>(splineTable.data))

def twophaseDarcy_incompressible_split_pp_sd_saturation_het_matType(int pskModelFlag,
                                                                    np.ndarray rowptr,
                                                                    np.ndarray colind,
                                                                    np.ndarray matType,
                                                                    double muw,
                                                                    double mun,
                                                                    np.ndarray omega,
                                                                    np.ndarray Kbar,
                                                                    double b,
                                                                    double capillaryDiffusionScaling,
                                                                    double advectionScaling,
                                                                    np.ndarray rwork_psk,
                                                                    np.ndarray iwork_psk,
                                                                    np.ndarray rwork_psk_tol,
                                                                    np.ndarray rwork_density_w,
                                                                    np.ndarray rwork_density_n,
                                                                    np.ndarray g, np.ndarray qt, np.ndarray u, np.ndarray sw, np.ndarray m, np.ndarray dm, np.ndarray phi, np.ndarray dphi,
                                                                    np.ndarray f, np.ndarray df, np.ndarray a, np.ndarray da):
    """Evaluate the saturation coefficients

Use the split fractional flow formulation of incompressible, two-phase
Darcy flow for a heterogeneous medium, sparse diffusion rep het
capillary pressure is primary variable

    """
    cdef int nPointsPerSimplex=sw.shape[sw.ndim-1]
    cdef int nSimplex=sw.size//nPointsPerSimplex
    if pskModelFlag == 0:
        tpdc.twophaseDarcy_incompressible_split_pp_sd_saturation_het_matType[SimplePSK](nSimplex,
                                                                                   nPointsPerSimplex,
                                                                                   g.shape[0],
                                                                                   2,
                                                                                   <int*>(rowptr.data),
                                                                                   <int*>(colind.data),
                                                                                   <int*>(matType.data),
                                                                                   muw,
                                                                                   mun,
                                                                                   <double*>(omega.data),
                                                                                   <double*>(Kbar.data),
                                                                                   b,
                                                                                   capillaryDiffusionScaling,advectionScaling,
                                                                                   <double*>(rwork_psk.data),<int*>(iwork_psk.data),
                                                                                   <double*>(rwork_psk_tol.data),
                                                                                   <double*>(rwork_density_w.data),
                                                                                   <double*>(rwork_density_n.data),
                                                                                   <double*>(g.data), <double*>(qt.data), <double*>(u.data), <double*>(sw.data), <double*>(m.data),
                                                                                   <double*>(dm.data), <double*>(phi.data), <double*>(dphi.data), <double*>(f.data),
                                                                                   <double*>(df.data), <double*>(a.data), <double*>(da.data))
    elif pskModelFlag == 1:
        tpdc.twophaseDarcy_incompressible_split_pp_sd_saturation_het_matType[VGM](nSimplex,
                                                                             nPointsPerSimplex,
                                                                             g.shape[0],
                                                                             4,
                                                                             <int*>(rowptr.data),
                                                                             <int*>(colind.data),
                                                                             <int*>(matType.data),
                                                                             muw,
                                                                             mun,
                                                                             <double*>(omega.data),
                                                                             <double*>(Kbar.data),
                                                                             b,
                                                                             capillaryDiffusionScaling,advectionScaling,
                                                                             <double*>(rwork_psk.data),<int*>(iwork_psk.data),
                                                                             <double*>(rwork_psk_tol.data),
                                                                             <double*>(rwork_density_w.data),
                                                                             <double*>(rwork_density_n.data),
                                                                             <double*>(g.data), <double*>(qt.data), <double*>(u.data), <double*>(sw.data), <double*>(m.data),
                                                                             <double*>(dm.data), <double*>(phi.data), <double*>(dphi.data), <double*>(f.data),
                                                                             <double*>(df.data), <double*>(a.data), <double*>(da.data))
    else:
        tpdc.twophaseDarcy_incompressible_split_pp_sd_saturation_het_matType[SimplePSK](nSimplex,
                                                                                   nPointsPerSimplex,
                                                                                   g.shape[0],
                                                                                   2,
                                                                                   <int*>(rowptr.data),
                                                                                   <int*>(colind.data),
                                                                                   <int*>(matType.data),
                                                                                   muw,
                                                                                   mun,
                                                                                   <double*>(omega.data),
                                                                                   <double*>(Kbar.data),
                                                                                   b,
                                                                                   capillaryDiffusionScaling,advectionScaling,
                                                                                   <double*>(rwork_psk.data),<int*>(iwork_psk.data),
                                                                                   <double*>(rwork_psk_tol.data),
                                                                                   <double*>(rwork_density_w.data),
                                                                                   <double*>(rwork_density_n.data),
                                                                                   <double*>(g.data), <double*>(qt.data), <double*>(u.data), <double*>(sw.data), <double*>(m.data),
                                                                                   <double*>(dm.data), <double*>(phi.data), <double*>(dphi.data), <double*>(f.data),
                                                                                   <double*>(df.data), <double*>(a.data), <double*>(da.data))
